#include "uyvy2rgb.h"

#include <QtCore/qmath.h>

UYVY2RGB::UYVY2RGB() : m_bits(NULL), m_width(0), m_height(0)
{
    int i;
    c298 = new int[256];
    for (i = 0; i < 256; i++)
        c298[i] = (i-16)*298;

    e409 = new int[256];
    e208 = new int[256];

    for (i = 0; i < 256; i++) {
        e409[i] = (i-128)*409;
        e208[i] = (i-128)*208;
    }

    d100 = new int[256];
    d516 = new int[256];

    for (i = 0; i < 256; i++) {
        d100[i] = (i-128)*100;
        d516[i] = (i-128)*516;
    }
}

UYVY2RGB::~UYVY2RGB()
{
    delete c298;
    delete e409;
    delete d100;
    delete e208;
    delete d516;
}

unsigned char* UYVY2RGB::bits()
{
    return m_bits;
}

int UYVY2RGB::width()
{
    return m_width;
}

int UYVY2RGB::height()
{
    return m_height;
}

void UYVY2RGB::initData(const QVideoFrame &source)
{
    if (m_bits == NULL || source.width() != m_width || source.height() != m_height)
    {
        if (m_bits != NULL)
            delete[] m_bits;

        m_width = source.width();
        m_height = source.height();
        m_bits = new unsigned char[m_width * m_height * 4];
    }

    int steps = (m_width/2)*m_height;

    unsigned char *dst = m_bits;
    const unsigned char *src = source.bits();

    for (int i = 0; i < steps; ++i)
    {
        int d   = *src    ; // u
        int y1  = *(src+1); // y
        int e   = *(src+2); // v
        int y2  = *(src+3); // y

        int rpart = e409[e] + 128;
        int gpart = d100[d] + e208[e] + 128;
        int bpart = d516[d] + 128;

        int r = (c298[y1] + rpart) >> 8;
        int g = (c298[y1] - gpart) >> 8;
        int b = (c298[y1] + bpart) >> 8;

        r = r < 0 ? 0 : (r > 255 ? 255 : r);
        g = g < 0 ? 0 : (g > 255 ? 255 : g);
        b = b < 0 ? 0 : (b > 255 ? 255 : b);

        *dst = b;
        *(dst+1) = g;
        *(dst+2) = r;
        *(dst+3) = 0;

        dst+=4;

        r = (c298[y2] + rpart) >> 8;
        g = (c298[y2] - gpart) >> 8;
        b = (c298[y2] + bpart) >> 8;

        r = r < 0 ? 0 : (r > 255 ? 255 : r);
        g = g < 0 ? 0 : (g > 255 ? 255 : g);
        b = b < 0 ? 0 : (b > 255 ? 255 : b);

        *dst = b;
        *(dst+1) = g;
        *(dst+2) = r;
        *(dst+3) = 0;

        dst+=4;
        src+=4;
    }
}
